/*******************************************************************************
Copyright 2016 Microchip Technology Inc. (www.microchip.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

To request to license the code under the MLA license (www.microchip.com/mla_license), 
please contact mla_licensing@microchip.com
*******************************************************************************/

/** INCLUDES *******************************************************/
#include "usb.h"
#include "usb_device_hid.h"

#include <string.h>

#include "system.h"


/** VARIABLES ******************************************************/
/* Some processors have a limited range of RAM addresses where the USB module
 * is able to access.  The following section is for those devices.  This section
 * assigns the buffers that need to be used by the USB module into those
 * specific areas.
 */
#if defined(FIXED_ADDRESS_MEMORY)
    #if defined(COMPILER_MPLAB_C18)
        #pragma udata HID_CUSTOM_OUT_DATA_BUFFER = HID_CUSTOM_OUT_DATA_BUFFER_ADDRESS
        unsigned char ReceivedDataBuffer[64];
        #pragma udata HID_CUSTOM_IN_DATA_BUFFER = HID_CUSTOM_IN_DATA_BUFFER_ADDRESS
        unsigned char ToSendDataBuffer[64];
        #pragma udata

    #elif defined(__XC8)
        unsigned char ReceivedDataBuffer[64] HID_CUSTOM_OUT_DATA_BUFFER_ADDRESS;
        unsigned char ToSendDataBuffer[64] HID_CUSTOM_IN_DATA_BUFFER_ADDRESS;
    #endif
#else
    unsigned char ReceivedDataBuffer[64];
    unsigned char ToSendDataBuffer[64];
#endif

//YTS volatile USB_HANDLE USBOutHandle;
//YTS volatile USB_HANDLE USBInHandle;

//YTS *****************************
volatile USB_HANDLE USBOutHandle1;
volatile USB_HANDLE USBInHandle1;
volatile USB_HANDLE USBOutHandle2;
volatile USB_HANDLE USBInHandle2;
//*********************************

/** DEFINITIONS ****************************************************/
typedef enum
{
    COMMAND_TOGGLE_LED = 0x80,
    COMMAND_GET_BUTTON_STATUS = 0x81,
    COMMAND_READ_POTENTIOMETER = 0x37
} CUSTOM_HID_DEMO_COMMANDS;

/** FUNCTIONS ******************************************************/

/*********************************************************************
* Function: void APP_DeviceCustomHIDInitialize(void);
*
* Overview: Initializes the Custom HID demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCustomHIDInitialize()
{
    //initialize the variable holding the handle for the last
    // transmission
    //YTS     USBInHandle = 0;

    USBInHandle1 = 0;
    USBInHandle2 = 0;

    //enable the HID endpoint
    //YTS    USBEnableEndpoint(CUSTOM_DEVICE_HID_EP, USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //YTS *****************************
    USBEnableEndpoint(CUSTOM_DEVICE_HID_EP1, USB_IN_ENABLED | USB_OUT_ENABLED | USB_HANDSHAKE_ENABLED | USB_DISALLOW_SETUP);
    USBEnableEndpoint(CUSTOM_DEVICE_HID_EP2, USB_IN_ENABLED | USB_OUT_ENABLED | USB_HANDSHAKE_ENABLED | USB_DISALLOW_SETUP);
    //*********************************

    //Re-arm the OUT endpoint for the next packet
    //YTS     USBOutHandle = (volatile USB_HANDLE)HIDRxPacket(CUSTOM_DEVICE_HID_EP,(uint8_t*)&ReceivedDataBuffer,64);
    //YTS *****************************
    USBOutHandle1 = (volatile USB_HANDLE)HIDRxPacket(CUSTOM_DEVICE_HID_EP1, (uint8_t*)&ReceivedDataBuffer[0],64);
    USBOutHandle2 = (volatile USB_HANDLE)HIDRxPacket(CUSTOM_DEVICE_HID_EP2, (uint8_t*)&ReceivedDataBuffer[0],64);
}

/*********************************************************************
* Function: void APP_DeviceCustomHIDTasks(void);
*
* Overview: Keeps the Custom HID demo running.
*
* PreCondition: The demo should have been initialized and started via
*   the APP_DeviceCustomHIDInitialize() and APP_DeviceCustomHIDStart() demos
*   respectively.
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCustomHIDTasks()
{   
    uint8_t i;
    /* If the USB device isn't configured yet, we can't really do anything
     * else since we don't have a host to talk to.  So jump back to the
     * top of the while loop. */
    if( USBGetDeviceState() < CONFIGURED_STATE )
    {
        return;
    }

    /* If we are currently suspended, then we need to see if we need to
     * issue a remote wakeup.  In either case, we shouldn't process any
     * keyboard commands since we aren't currently communicating to the host
     * thus just continue back to the start of the while loop. */
    if( USBIsDeviceSuspended()== true )
    {
        return;
    }
    
    //Check if we have received an OUT data packet from the host
    if(HIDRxHandleBusy(USBOutHandle1) == false)
    {   
        //We just received a packet of data from the USB host.
        //Check to make sure the endpoint/buffer is free before we modify the contents
        if (!HIDTxHandleBusy(USBInHandle1)) {
            strcpy(ToSendDataBuffer, ReceivedDataBuffer);
            i = ReceivedDataBuffer[0];
            ToSendDataBuffer[0] += 1;

            //Prepare the USB module to send the data packet to the host
            USBInHandle1 = HIDTxPacket(CUSTOM_DEVICE_HID_EP1, (uint8_t*) & ToSendDataBuffer[0], 64);
        }
        //Re-arm the OUT endpoint, so we can receive the next OUT data packet 
        //that the host may try to send us.
        USBOutHandle1 = HIDRxPacket(CUSTOM_DEVICE_HID_EP1, (uint8_t*) & ReceivedDataBuffer, 64);
    }


    //Check if we have received an OUT data packet from the host
    if (HIDRxHandleBusy(USBOutHandle2) == false) {//HID Generic
        //We just received a packet of data from the USB host.

        //Check to make sure the endpoint/buffer is free before we modify the contents
        if (!HIDTxHandleBusy(USBInHandle2)) {
            strcpy(ToSendDataBuffer, ReceivedDataBuffer);
            i = ReceivedDataBuffer[0];
            ToSendDataBuffer[0] += 2;
            //Prepare the USB module to send the data packet to the host
            USBInHandle2 = HIDTxPacket(CUSTOM_DEVICE_HID_EP2, (uint8_t*) & ToSendDataBuffer[0], 64);
        }

        //Re-arm the OUT endpoint, so we can receive the next OUT data packet
        //that the host may try to send us.
        USBOutHandle2 = HIDRxPacket(CUSTOM_DEVICE_HID_EP2, (uint8_t*)&ReceivedDataBuffer[0], 64);
    }
}